@@ -7,7 +7,11 @@ This file is licensed to you under the license specified in the included file
 /*
   Called by yyparse on error.
  */
+
+#include "yyerror.h" 
+
 #include <stdio.h>
+#include <iostream>
 
 extern int gengetopt_count_line;
 extern char * gengetopt_input_filename;
@@ -16,6 +20,8 @@ extern int tokenpos;
 extern char linebuf[];
 extern char *yytext;
 
+using namespace std;
+
 void
 yyerror (const char *s)
 {
@@ -28,14 +34,11 @@ yyerror (const char *s)
            tokenpos + 1, "^");
 }
 
-// this second version should be used when the line has already been parsed,
-// but there's a "semantic" error.  Indeed the line number has to be
-// decremented.
 void
-yyerror2 (const char *s)
+yyerror (gengetopt_option *opt, const char *s)
 {
   const char *source =
-    (gengetopt_input_filename ? gengetopt_input_filename : "gengetopt");
+    (opt->filename ? opt->filename : "gengetopt");
 
-  fprintf (stderr, "%s:%d: %s\n", source, gengetopt_count_line - 1, s);
+  cerr << source << ":" << opt->linenum << ": " << s << endl;
 }
