@@ -23,6 +23,7 @@ main (int argc, char **argv)
   if (args_info.string_values_given)
     {
       printf ("specified --string-values with argument %s\n", args_info.string_values_arg);
+      printf ("specified --string-values with (original) argument %s\n", args_info.string_values_orig);
     }
 
   printf ("specified --string-values-def with argument %s\n", args_info.string_values_def_arg);
@@ -32,8 +33,11 @@ main (int argc, char **argv)
       printf ("specified --string-values-no-short with argument %s\n", args_info.string_values_no_short_arg);
     }
 
-  for (i = 0; i < args_info.multistring_values_given; ++i)
-    printf ("specified --multistring-values with argument %s\n", args_info.multistring_values_arg[i]);
+  for (i = 0; i < args_info.multistring_values_given; ++i) 
+    {
+      printf ("specified --multistring-values with argument %s\n", args_info.multistring_values_arg[i]);
+      printf ("specified --multistring-values with (original) argument %s\n", args_info.multistring_values_orig[i]);
+    }
 
   for (i = 0; i < args_info.multistring_values_no_short_given; ++i)
     printf ("specified --multistring-values-no-short with argument %s\n", args_info.multistring_values_no_short_arg[i]);
